SuperStrict
Rem
	bbdoc: A flexible animation module
	about: By using this module, you can easily animate every value by using pointers.
	It is frame-independent (If you call #UpdateAnimations oftener, the animations will have more steps
	but not be faster)
End Rem
Module ZEVS.animation
ModuleInfo "Author: <a href='Felix.Schremmer@gmx.de'>Felix Schremmer</a> alias ZEVS@www.blitzforum.de"
ModuleInfo "Copyright: Felix Schremmer 2011"
ModuleInfo "Liscense: General Public Liscense (http://www.gnu.org/copyleft/gpl.html)"
ModuleInfo "Version: 1.0b"
ModuleInfo "Else: If you use this module, it would be nice to write it in your credits"

Import BRL.Math
Import BRL.LinkedList
Import BRl.Reflection


Public
Rem
	bbdoc: The animation-type.
End Rem
Type TAnimation

  Field value:TAnimationValue, start:TAnimationNumber, aim:TAnimationNumber
  Field creation% = MilliSecs(), length%

  Field Data:Object

  Field trigger:TAnimationTrigger, voidTrigger()

  Global list:TList = New TList
  Method New()
    Self.list.addLast Self
  End Method
  Method Destroy()
    Self.list.remove Self
  End Method
  Function CreateForTime:TAnimation(value:TAnimationValue, aim:TAnimationNumber, time%)
    Local result:TAnimation = New TAnimation
    result.value = value
    result.start = TAnimationNumber.fromValue(value)
    result.aim = aim
    result.length = time
    Return result
  End Function
  Function CreateForStep:TAnimation(value:TAnimationValue, aim:TAnimationNumber, stp:TAnimationNumber)
    Local time% = TAnimationNumber.fromValue(value).getTime(aim, stp, value.format)
    Return Self.CreateForTime(value, aim, time)
  End Function
  Function LongCreateForTime:TAnimation(value:Long Ptr, aim:Long, time%)
    Return Self.CreateForTime(TAnimationValue.FromLong(value), TAnimationNumber.FromLong(aim), time)
  End Function
  Function IntCreateForTime:TAnimation(value:Int Ptr, aim:Int, time%)
    Return Self.CreateForTime(TAnimationValue.FromInt(value), TAnimationNumber.FromInt(aim), time)
  End Function
  Function ShortCreateForTime:TAnimation(value:Short Ptr, aim:Short, time%)
    Return Self.CreateForTime(TAnimationValue.FromShort(value), TAnimationNumber.FromShort(aim), time)
  End Function
  Function ByteCreateForTime:TAnimation(value:Byte Ptr, aim:Byte, time%)
    Return Self.CreateForTime(TAnimationValue.FromByte(value), TAnimationNumber.FromByte(aim), time)
  End Function
  Function DoubleCreateForTime:TAnimation(value:Double Ptr, aim:Double, time%)
    Return Self.CreateForTime(TAnimationValue.FromDouble(value), TAnimationNumber.FromDouble(aim), time)
  End Function
  Function FloatCreateForTime:TAnimation(value:Float Ptr, aim:Float, time%)
    Return Self.CreateForTime(TAnimationValue.FromFloat(value), TAnimationNumber.FromFloat(aim), time)
  End Function
  Function LongCreateForStep:TAnimation(value:Long Ptr, aim:Long, stp:Long)
    Return Self.CreateForStep(TAnimationValue.FromLong(value), TAnimationNumber.FromLong(aim),..
                                    TAnimationNumber.fromLong(stp))
  End Function
  Function IntCreateForStep:TAnimation(value:Int Ptr, aim:Int, stp:Int)
    Return Self.CreateForStep(TAnimationValue.FromInt(value), TAnimationNumber.FromInt(aim),..
                                       TAnimationNumber.fromInt(stp))
  End Function
  Function ShortCreateForStep:TAnimation(value:Short Ptr, aim:Short, stp:Short)
    Return Self.CreateForStep(TAnimationValue.FromShort(value), TAnimationNumber.FromShort(aim),..
                                       TAnimationNumber.fromShort(stp))
  End Function
  Function ByteCreateForStep:TAnimation(value:Byte Ptr, aim:Byte, stp:Byte)
    Return Self.CreateForStep(TAnimationValue.FromByte(value), TAnimationNumber.FromByte(aim),..
                                       TAnimationNumber.fromByte(stp))
  End Function
  Function DoubleCreateForStep:TAnimation(value:Double Ptr, aim:Double, stp:Double)
    Return Self.CreateForStep(TAnimationValue.FromDouble(value), TAnimationNumber.FromDouble(aim),..
                                       TAnimationNumber.fromDouble(stp))
  End Function
  Function FloatCreateForStep:TAnimation(value:Float Ptr, aim:Float, stp:Float)
    Return Self.CreateForStep(TAnimationValue.FromFloat(value), TAnimationNumber.FromFloat(aim),..
                                       TAnimationNumber.fromFloat(stp))
  End Function

  Method setVoidTrigger(trigger())
    Self.voidTrigger = trigger
    Self.trigger = Null
  End Method
  Method setTrigger(trigger:TAnimationTrigger)
    Self.voidTrigger = Null
    Self.trigger = trigger
  End Method

  Method Update()
    Local time% = MilliSecs()-Self.creation
    If time > Self.length Then
      Self.setToEnd
      Return
    EndIf
    Local rel! = Double(time)/Double(Self.length)
    Self.setStep rel
  End Method
  Method SetStep(rel!)
    Self.value.update Self.start, Self.aim, rel
  End Method
  Method SetToEnd()
    Self.value.setToAim Self.aim
    Self.destroy
    If Self.voidTrigger Then
      Self.voidTrigger
    ElseIf Self.trigger Then
      Self.value.callTrigger Self.trigger
    EndIf
  End Method
  Method Cancel()
    Self.destroy
  End Method
  Function UpdateAll()
    For Local animation:TAnimation = EachIn Self.list
      animation.update
    Next
  End Function
End Type
Private
Type TAnimationValue
  Field valueLong:Long Ptr, valueInt:Int Ptr, valueShort:Short Ptr, valueByte:Byte Ptr
  Field valueDouble:Double Ptr, valueFloat:Float Ptr
  Field format:TTypeId
  Method update(start:TAnimationNumber, aim:TAnimationNumber, rel!)
    Select Self.format
     Case LongTypeId
      Self.valueLong[0] = start.valueLong+(aim.valueLong-start.valueLong)*rel
     Case IntTypeId
      Self.valueInt[0] = start.valueInt+(aim.valueInt-start.valueInt)*rel
     Case ShortTypeId
      Self.valueShort[0] = start.valueShort+(aim.valueShort-start.valueShort)*rel
     Case ByteTypeId
      Self.valueByte[0] = start.valueByte+(aim.valueByte-start.valueByte)*rel
     Case DoubleTypeId
      Self.valueDouble[0] = start.valueDouble+(aim.valueDouble-start.valueDouble)*rel
     Case FloatTypeId
      Self.valueFloat[0] = start.valueFloat+(aim.valueFloat-start.valueFloat)*rel
    End Select
  End Method
  Function fromLong:TAnimationValue(value:Long Ptr)
    Local result:TAnimationValue = New TAnimationValue
    result.format = LongTypeId
    result.valueLong = value 
    Return result
  End Function
  Function fromInt:TAnimationValue(value:Int Ptr)
    Local result:TAnimationValue = New TAnimationValue
    result.format = IntTypeId
    result.valueInt = value 
    Return result
  End Function
  Function fromShort:TAnimationValue(value:Short Ptr)
    Local result:TAnimationValue = New TAnimationValue
    result.format = ShortTypeId
    result.valueShort = value 
    Return result
  End Function
  Function fromByte:TAnimationValue(value:Byte Ptr)
    Local result:TAnimationValue = New TAnimationValue
    result.format = ByteTypeId
    result.valueByte = value 
    Return result
  End Function
  Function fromDouble:TAnimationValue(value:Double Ptr)
    Local result:TAnimationValue = New TAnimationValue
    result.format = DoubleTypeId
    result.valueDouble = value 
    Return result
  End Function
  Function fromFloat:TAnimationValue(value:Float Ptr)
    Local result:TAnimationValue = New TAnimationValue
    result.format = FloatTypeId
    result.valueFloat = value 
    Return result
  End Function

  Method setToAim(aim:TAnimationNumber)
    Select Self.format
     Case LongTypeId
      Self.valueLong[0] = aim.valueLong
     Case IntTypeId
      Self.valueInt[0] = aim.valueInt
     Case ShortTypeId
      Self.valueShort[0] = aim.valueShort
     Case ByteTypeId
      Self.valueByte[0] = aim.valueByte
     Case DoubleTypeId
      Self.valueDouble[0] = aim.valueDouble
     Case FloatTypeId
      Self.valueFloat[0] = aim.valueFloat
    End Select
  End Method

  Method callTrigger(trigger:TAnimationTrigger)
    Select Self.format
     Case LongTypeId
      If trigger.valueLong   Then trigger.valueLong(  Self.valueLong  )
     Case IntTypeId
      If trigger.valueInt    Then trigger.valueInt(   Self.valueInt   )
     Case ShortTypeId
      If trigger.valueShort  Then trigger.valueShort( Self.valueShort )
     Case ByteTypeId
      If trigger.valueByte   Then trigger.valueByte(  Self.valueByte  )
     Case DoubleTypeId
      If trigger.valueDouble Then trigger.valueDouble(Self.valueDouble)
     Case FloatTypeId
      If trigger.valueFloat  Then trigger.valueFloat( Self.valueFloat )
    End Select
  End Method
End Type
Type TAnimationNumber
  Field valueLong:Long, valueInt:Int, valueShort:Short, valueByte:Byte
  Field valueDouble:Double, valueFloat:Float
  Function fromLong:TAnimationNumber(value:Long)
    Local result:TAnimationNumber = New TAnimationNumber
    result.valueLong = value 
    Return result
  End Function
  Function fromInt:TAnimationNumber(value:Int)
    Local result:TAnimationNumber = New TAnimationNumber
    result.valueInt = value 
    Return result
  End Function
  Function fromShort:TAnimationNumber(value:Short)
    Local result:TAnimationNumber = New TAnimationNumber
    result.valueShort = value 
    Return result
  End Function
  Function fromByte:TAnimationNumber(value:Byte)
    Local result:TAnimationNumber = New TAnimationNumber
    result.valueByte = value 
    Return result
  End Function
  Function fromDouble:TAnimationNumber(value:Double)
    Local result:TAnimationNumber = New TAnimationNumber
    result.valueDouble = value 
    Return result
  End Function
  Function fromFloat:TAnimationNumber(value:Float)
    Local result:TAnimationNumber = New TAnimationNumber
    result.valueFloat = value
    Return result
  End Function
  Function fromValue:TAnimationNumber(value:TAnimationValue)
    Local result:TAnimationNumber = New TAnimationNumber
    Select value.format
     Case LongTypeId
      result.valueLong = value.valueLong[0]
     Case IntTypeId
      result.valueInt = value.valueInt[0]
     Case ShortTypeId
      result.valueShort = value.valueShort[0]
     Case ByteTypeId
      result.valueByte = value.valueByte[0]
     Case DoubleTypeId
      result.valueDouble = value.valueDouble[0]
     Case FloatTypeId
      result.valueFloat = value.valueFloat[0]
    End Select
    Return result
  End Function
  Method getTime%(aim:TAnimationNumber, stp:TAnimationNumber, format:TTypeId)
    Select format
     Case LongTypeId
      Return Abs(Self.valueLong-aim.valueLong)/stp.valueLong
     Case IntTypeId
      Return Abs(Self.valueInt-aim.valueInt)/stp.valueInt
     Case ShortTypeId
      Return Abs(Self.valueShort-aim.valueShort)/stp.valueShort
     Case ByteTypeId
      Return Abs(Self.valueByte-aim.valueByte)/stp.valueByte
     Case DoubleTypeId
      Return Abs(Self.valueDouble-aim.valueDouble)/stp.valueDouble
     Case FloatTypeId
      Return Abs(Self.valueFloat-aim.valueFloat)/stp.valueFloat
    End Select
  End Method
End Type
Public
Rem
	bbdoc: The Type storing animation triggers
End Rem
Type TAnimationTrigger
  Field valueLong(value:Long Ptr), valueInt(value:Int Ptr),       valueShort(value:Short Ptr)
  Field valueByte(value:Byte Ptr), valueDouble(value:Double Ptr), valueFloat(value:Float Ptr)
  Function forLong:TAnimationTrigger(func(value:Long Ptr))
    Local result:TAnimationTrigger = New TAnimationTrigger
    result.valueLong = func
    Return result
  End Function
  Function forInt:TAnimationTrigger(func(value:Int Ptr))
    Local result:TAnimationTrigger = New TAnimationTrigger
    result.valueInt = func
    Return result
  End Function
  Function forShort:TAnimationTrigger(func(value:Short Ptr))
    Local result:TAnimationTrigger = New TAnimationTrigger
    result.valueShort = func
    Return result
  End Function
  Function forByte:TAnimationTrigger(func(value:Byte Ptr))
    Local result:TAnimationTrigger = New TAnimationTrigger
    result.valueByte = func
    Return result
  End Function
  Function forDouble:TAnimationTrigger(func(value:Double Ptr))
    Local result:TAnimationTrigger = New TAnimationTrigger
    result.valueDouble = func
    Return result
  End Function
  Function forFloat:TAnimationTrigger(func(value:Float Ptr))
    Local result:TAnimationTrigger = New TAnimationTrigger
    result.valueFloat = func
    Return result
  End Function
End Type
Rem
	bbdoc: animates a long pointer
	returns: The animation-Object.
	about: The first argument (value) is a long-pointer (See VarPtr).<br>
	The secound argument is the animation's aim.<br>
	The third argument is the time (in ms), until the animation reaches the aim.<br>
	The fourth argument is optional. It can be an object, associated with the animation to avoid
	the value be unset and the pointer become invalid.<br><br>
	NOTE: You cannot animate in the whole long-space. That's why animation-steps are performed as
	doubles, which have not the same space.
End Rem
Function AnimateLong:TAnimation(value:Long Ptr, aim:Long, time%, data:Object = Null)
  Local result:TAnimation = TAnimation.LongCreateForTime(value, aim, time)
  result.data = data
  Return result
End Function
Rem
	bbdoc: animates an int pointer
	returns: The animation-Object.
	about: The first argument (value) is an int-pointer (See VarPtr).<br>
	The secound argument is the animation's aim.<br>
	The third argument is the time (in ms), until the animation reaches the aim.<br>
	The fourth argument is optional. It can be an object, associated with the animation to avoid
	the value be unset and the pointer become invalid.
End Rem
Function AnimateInt:TAnimation(value:Int Ptr, aim:Int, time%, data:Object = Null)
  Local result:TAnimation = TAnimation.IntCreateForTime(value, aim, time)
  result.data = data
  Return result
End Function
Rem
	bbdoc: animates a short pointer
	returns: The animation-Object.
	about: The first argument (value) is a short-pointer (See VarPtr).<br>
	The secound argument is the animation's aim.<br>
	The third argument is the time (in ms), until the animation reaches the aim.<br>
	The fourth argument is optional. It can be an object, associated with the animation to avoid
	the value be unset and the pointer become invalid.
End Rem
Function AnimateShort:TAnimation(value:Short Ptr, aim:Short, time%, data:Object = Null)
  Local result:TAnimation = TAnimation.ShortCreateForTime(value, aim, time)
  result.data = data
  Return result
End Function
Rem
	bbdoc: animates a byte pointer
	returns: The animation-Object.
	about: The first argument (value) is a byte-pointer (See VarPtr).<br>
	The secound argument is the animation's aim.<br>
	The third argument is the time (in ms), until the animation reaches the aim.<br>
	The fourth argument is optional. It can be an object, associated with the animation to avoid
	the value be unset and the pointer become invalid.
End Rem
Function AnimateByte:TAnimation(value:Byte Ptr, aim:Byte, time%, data:Object = Null)
  Local result:TAnimation = TAnimation.ByteCreateForTime(value, aim, time)
  result.data = data
  Return result
End Function
Rem
	bbdoc: animates a double pointer
	returns: The animation-Object.
	about: The first argument (value) is a double-pointer (See VarPtr).<br>
	The secound argument is the animation's aim.<br>
	The third argument is the time (in ms), until the animation reaches the aim.<br>
	The fourth argument is optional. It can be an object, associated with the animation to avoid
	the value be unset and the pointer become invalid.
End Rem
Function AnimateDouble:TAnimation(value:Double Ptr, aim:Double, time%, data:Object = Null)
  Local result:TAnimation = TAnimation.DoubleCreateForTime(value, aim, time)
  result.data = data
  Return result
End Function
Rem
	bbdoc: animates a float pointer
	returns: The animation-Object.
	about: The first argument (value) is a float-pointer (See VarPtr).<br>
	The secound argument is the animation's aim.<br>
	The third argument is the time (in ms), until the animation reaches the aim.<br>
	The fourth argument is optional. It can be an object, associated with the animation to avoid
	the value be unset and the pointer become invalid.
End Rem
Function AnimateFloat:TAnimation(value:Float Ptr, aim:Float, time%, data:Object = Null)
  Local result:TAnimation = TAnimation.FloatCreateForTime(value, aim, time)
  result.data = data
  Return result
End Function


Rem
	bbdoc: animates a long pointer
	returns: The animation-Object.
	about: The first argument (value) is a long-pointer (See VarPtr).<br>
	The secound argument is the animation's aim.<br>
	The third argument is the step, how much the value will change in one ms.<br>
	The fourth argument is optional. It can be an object, associated with the animation to avoid
	the value be unset and the pointer to become invalid.<br><br>
	NOTE: You cannot animate in the whole long-space. That's why animation-steps are performed as
	doubles, which have not the same space.
End Rem
Function AnimateLongByStep:TAnimation(value:Long Ptr, aim:Long, stp:Long, data:Object = Null)
  Local result:TAnimation = TAnimation.LongCreateForStep(value, aim, stp)
  result.data = data
  Return result
End Function
Rem
	bbdoc: animates an int pointer
	returns: The animation-Object.
	about: The first argument (value) is an int-pointer (See VarPtr).<br>
	The secound argument is the animation's aim.<br>
	The third argument is the step, how much the value will change in one ms.<br>
	The fourth argument is optional. It can be an object, associated with the animation to avoid
	the value be unset and the pointer to become invalid.
End Rem
Function AnimateIntByStep:TAnimation(value:Int Ptr, aim:Int, stp:Int, data:Object = Null)
  Local result:TAnimation = TAnimation.IntCreateForStep(value, aim, stp)
  result.data = data
  Return result
End Function
Rem
	bbdoc: animates a short pointer
	returns: The animation-Object.
	about: The first argument (value) is a short-pointer (See VarPtr).<br>
	The secound argument is the animation's aim.<br>
	The third argument is the step, how much the value will change in one ms.<br>
	The fourth argument is optional. It can be an object, associated with the animation to avoid
	the value be unset and the pointer to become invalid.
End Rem
Function AnimateShortByStep:TAnimation(value:Short Ptr, aim:Short, stp:Short, data:Object = Null)
  Local result:TAnimation = TAnimation.ShortCreateForStep(value, aim, stp)
  result.data = data
  Return result
End Function
Rem
	bbdoc: animates a byte pointer
	returns: The animation-Object.
	about: The first argument (value) is a byte-pointer (See VarPtr).<br>
	The secound argument is the animation's aim.<br>
	The third argument is the step, how much the value will change in one ms.<br>
	The fourth argument is optional. It can be an object, associated with the animation to avoid
	the value be unset and the pointer to become invalid.
End Rem
Function AnimateByteByStep:TAnimation(value:Byte Ptr, aim:Byte, stp:Byte, data:Object = Null)
  Local result:TAnimation = TAnimation.ByteCreateForStep(value, aim, stp)
  result.data = data
  Return result
End Function
Rem
	bbdoc: animates a double pointer
	returns: The animation-Object.
	about: The first argument (value) is a double-pointer (See VarPtr).<br>
	The secound argument is the animation's aim.<br>
	The third argument is the step, how much the value will change in one ms.<br>
	The fourth argument is optional. It can be an object, associated with the animation to avoid
	the value be unset and the pointer to become invalid.
End Rem
Function AnimateDoubleByStep:TAnimation(value:Double Ptr, aim:Double, stp:Double, data:Object = Null)
  Local result:TAnimation = TAnimation.DoubleCreateForStep(value, aim, stp)
  result.data = data
  Return result
End Function
Rem
	bbdoc: animates a double pointer
	returns: The animation-Object.
	about: The first argument (value) is a double-pointer (See VarPtr).<br>
	The secound argument is the animation's aim.<br>
	The third argument is the step, how much the value will change in one ms.<br>
	The fourth argument is optional. It can be an object, associated with the animation to avoid
	the value be unset and the pointer to become invalid.
End Rem
Function AnimateFloatByStep:TAnimation(value:Float Ptr, aim:Float, stp:Float, data:Object = Null)
  Local result:TAnimation = TAnimation.FloatCreateForStep(value, aim, stp)
  result.data = data
  Return result
End Function

Rem
	bbdoc: clears all animations
	returns: void
	about: Use this to remove all animations from the update-list.
End Rem
Function ClearAnimations()
  TAnimation.list.Clear
End Function
Rem
	bbdoc: cancels the animation
	returns: void
	about: Removes the animation from the update-list. The value will NOT be changed.
End Rem
Function CancelAnimation(animation:TAnimation)
  animation.cancel
End Function
Rem
	bbdoc: ends the animation
	returns: void
	about: Sets the value to the aim and removes the animation from the update-list
End Rem
Function EndAnimation(animation:TAnimation)
  animation.setToEnd
End Function
Rem
	bbdoc: sets an animation-step
	returns: void
	about: Sets the value to the step associated with stp (Double between 1 and 0). 0 means, the value
	will be set to the start and 1 means, the value will be set to the end. It doesn't affect any updates -
	the next update will overwrite the step. To avoid updates, use PauseAnimation.
End Rem
Function SetAnimationStep(animation:TAnimation, stp!)
  animation.setStep stp
End Function
Rem
	bbdoc: updates all animations.
	returns: void
	about: Updates all the animations of the update-list.
End Rem
Function UpdateAnimations()
  TAnimation.updateAll
End Function
Rem
	bbdoc: pauses the animation.
	returns: void
	about: Removes the animation from the update-list.
End Rem
Function PauseAnimation(animation:TAnimation)
  TAnimation.list.remove animation
End Function
Rem
	bbdoc: resumes the animation.
	returns: void
	about: Re-adds the animation to the update-list.
End Rem
Function ResumeAnimation(animation:TAnimation)
  TAnimation.list.addLast animation
End Function
Rem
	bbdoc: Sets a trigger for animation completition.
	returns: void
	about: Sets a trigger, which will be called when the animation finishes.<br>
	It will delete any trigger set before.<br>
	If your function has no arguments, use #SetAnimationVoidTrigger.<br>
	The function will be called with the variable-pointer.
End Rem
Function SetAnimationTrigger(animation:TAnimation, trigger:TAnimationTrigger)
  animation.setTrigger trigger
End Function
Rem
	bbdoc: Sets a void trigger for animation completition.
	returns: void
	about: Same to #SetAnimationTrigger, but here is a function-pointer directly given. Use this,
	if your function expects no arguments.
End Rem
Function SetAnimationVoidTrigger(animation:TAnimation, trigger())
  animation.setVoidTrigger trigger
End Function
Rem
	bbdoc: Creates a trigger for Long animations.
	returns: The created trigger
	about: Combine this function with #SetAnimationTrigger. The function will be called with the
	animated pointer. Use #SetAnimationVoidTrigger, if you want no arguments.
End Rem
Function LongAnimationTrigger:TAnimationTrigger(trigger(value:Long Ptr))
  Return TAnimationTrigger.forLong(trigger)
End Function
Rem
	bbdoc: Creates a trigger for Int animations.
	returns: The created trigger
	about: Combine this function with #SetAnimationTrigger. The function will be called with the
	animated pointer. Use #SetAnimationVoidTrigger, if you want no arguments.
End Rem
Function IntAnimationTrigger:TAnimationTrigger(trigger(value:Int Ptr))
  Return TAnimationTrigger.forInt(trigger)
End Function
Rem
	bbdoc: Creates a trigger for Short animations.
	returns: The created trigger
	about: Combine this Function with #SetAnimationTrigger. The Function will be called with the
	animated pointer. Use #SetAnimationVoidTrigger, If you want no arguments.
End Rem
Function ShortAnimationTrigger:TAnimationTrigger(trigger(value:Short Ptr))
  Return TAnimationTrigger.forShort(trigger)
End Function
Rem
	bbdoc: Creates a trigger for Byte animations.
	returns: The created trigger
	about: Combine this Function with #SetAnimationTrigger. The Function will be called with the
	animated pointer. Use #SetAnimationVoidTrigger, If you want no arguments.
End Rem
Function ByteAnimationTrigger:TAnimationTrigger(trigger(value:Byte Ptr))
  Return TAnimationTrigger.forByte(trigger)
End Function
Rem
	bbdoc: Creates a trigger for Double animations.
	returns: The created trigger
	about: Combine this Function with #SetAnimationTrigger. The Function will be called with the
	animated pointer. Use #SetAnimationVoidTrigger, If you want no arguments.
End Rem
Function DoubleAnimationTrigger:TAnimationTrigger(trigger(value:Double Ptr))
  Return TAnimationTrigger.forDouble(trigger)
End Function
Rem
	bbdoc: Creates a trigger for Float animations.
	returns: The created trigger
	about: Combine this Function with #SetAnimationTrigger. The Function will be called with the
	animated pointer. Use #SetAnimationVoidTrigger, If you want no arguments.
End Rem
Function FloatAnimationTrigger:TAnimationTrigger(trigger(value:Float Ptr))
  Return TAnimationTrigger.forFloat(trigger)
End Function